from enum import Enum


class MessageType(Enum):
    REPLY = 0
    AUDIO = 1
    DOCUMENT = 2
    ANIMATION = 3
    IMAGE = 4
    STICKER = 5
    VIDEO = 6
    VOICE = 7
    VIDEO_NOTE = 8
    CONTACT = 9
    LOCATION = 10
    VENUE = 11
    TEXT = 12
    COMMAND = 13



from pvlv.handlers import base_handler
from pvlv.handlers import img_handler
from pvlv.handlers import text_handler
from pvlv.handlers import vocal_handler

from pvlv.pvlv import BotInterface

from pvlv.debug.execution_time import ExecutionTime

from pvlv.static import configurations
from pvlv.static import settings
from pvlv.static import types



from io import BytesIO
from pydub import AudioSegment
import speech_recognition as sr
from pvlv.static.settings import *
from pvlv.pvlv import BotInterface


def speech_to_text(bot: BotInterface, file, guild_id, language):

    if language != ITA:
        language = 'it-IT'
    else:
        language = 'en-EN'

    """
    Download the telegram file
    And prepare it for manipulation
    """
    raw_file = BytesIO()  # the file download needs a var where write the audio (on ram)
    raw_file = file.download(out=raw_file)
    raw_file.seek(0)  # reset the current position of the segment

    """
    Speed up the audio for a faster upload
    - create a new frame rate
    - convert the audio
    - set the frame rate for export, reducing the quality for a more light weight file
    """
    audio_segment = AudioSegment.from_ogg(raw_file)
    audio_segment = audio_segment.set_frame_rate(10000)  # reduce the quality of the audio for a faster upload
    audio = audio_segment.export(format='wav')
    audio.seek(0)

    """
    Open the segment as Audio file obj,
    and convert to text the content
    """
    r = sr.Recognizer()
    with sr.AudioFile(audio) as source:
        audio_data = r.record(source)  # read the entire audio file
        r.pause_threshold = 4.0

    try:
        text = r.recognize_google(audio_data, language=language)
    except Exception as e:
        print(e)
        text = ""

    # finally sent the transcription to the chat
    bot.reply_text(guild_id, 'Speech to text:\n{}'.format(text))



import configparser as cfg
import logging


CONFIG_PATH = 'configs/pavlov.cfg'

parser = cfg.ConfigParser()
try:
    parser.read(CONFIG_PATH)
except Exception as exc:
    print(exc)

__beta = parser.getboolean('credentials', 'BETA', fallback=False)
__TOKEN = parser.get('credentials', 'TOKEN', fallback='')
__TOKEN_BETA = parser.get('credentials', 'TOKEN_BETA', fallback='')
TOKEN = __TOKEN_BETA if __beta else __TOKEN

OWNER_ID = parser.get('admin', 'OWNER_ID', fallback=111111111)
LOGGING_CHAT = parser.get('admin', 'LOGGING_CHAT', fallback=111111111)

DEBUG = parser.get('debug', 'DEBUG', fallback=False)
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger('pavlov')

if DEBUG:
    logger.setLevel(logging.INFO)
else:
    logger.setLevel(logging.ERROR)




class BotInterface:
    def __init__(self):

        self.user_id = None
        self.username = None

        self.guild_id = None
        self.guild_name = None

        self.timestamp = None

    def send_text(self, location, text, parse_mode=False):
        pass

    def reply_text(self, text, parse_mode=False):
        pass

    def send_photo(self, location, picture):
        pass

    def reply_photo(self, picture):
        pass

    def send_file(self, file):
        pass



from pvlv_commando.languages.languages_handler import language_selector


def user_field(language, user_name):

    def eng(): return 'Cool {}'.format(user_name)
    def ita(): return 'Grande {}'.format(user_name)

    return language_selector(
        language,
        eng, ita=ita
    )


def text_description(language, level):

    def eng(): return 'You\'ve gain to level {}'.format(level)
    def ita(): return 'Hai raggiunto il livello {}'.format(level)

    return language_selector(
        language,
        eng, ita=ita
    )



# base
ENABLED = 1
DISABLED = 0
ERROR = -1


# Languages Handled for messages
ENG = 'eng'
ITA = 'ita'

# message max length for analyze
MEX_MAX_LENGTH = 70


# situational reply settings
AVOID_REPLY = 0
STD_REPLAY = 1
POWER_REPLAY = 2
STATIC_REPLAY = 3
STATIC_SPLIT_KEY = '&&&'
STATIC_SPLIT_MODE = '%%%'
STATIC_OVERRIDE_MODE = '!'


# CONFIGURATION SETTINGS
# prefix type
NO_PREFIX = -1
COMMAND_PREFIX = 0
OVERRIDE_PREFIX = 1
SUDO_PREFIX = 2

# global setting for auto reply _commands
DISABLED_MODE = 0
QUIET_MODE = 1
NORMAL_MODE = 2
SPAM_MODE = 3
AGGRESSIVE_MODE = 4

# module type, to define the _scope of the module
TYPE_LISTENER = 0
TYPE_COMMAND = 1


# ERRORS CONFIG
# message_reply_error
WRONG_STATIC_MODE_STRING = 1



from pvlv.handlers.base_handler import BaseHandler


class TextHandler(BaseHandler):
    """
    It will handle the operations on the text.
    """
    def __init__(self, bot_interface):
        super().__init__(bot_interface)

    def handle(self, text):

        self.stats_updater.update_text(text)  # update stats

        if not self.check_commands(text):
            self.check_spam()

        self.check_level_up()



from pvlv_commando.languages.languages_handler import language_selector


def spam_detected_reply(language):

    def eng(): return 'Stop spamming.'
    def ita(): return 'Basta spammare.'

    return language_selector(
        language,
        eng, ita=ita
    )



from pvlv.pvlv import BotInterface

from pvlv.static.configurations import logger, LOGGING_CHAT
from pvlv_database import Database, StatsUpdater
from pvlv_img_builder.level_up_card import DrawLevelUpCard
from pvlv.handlers.level_up.level_up_reply import user_field, text_description
from pvlv.handlers.antispam.antispam_reply import spam_detected_reply
from pvlv_commando import (
    CommandNotFound,
    CommandExecutionFail,
    ArgVoidNotAllowed,
    ParamNotFound,
    MaxHourlyUses,
    MaxDailyUses,
)
from pvlv_commando import Commando

com = Commando()  # load all the the commands in the commands dir


class BaseHandler(object):
    """
    Middle point to prepare data that have to be saved to the database
    This is an interface between the platform-wrapper and the other modules

    This Handler will handle the operations on the text.
     - stats updating
     - command reading
     - text interactions
    """
    def __init__(self, bot_interface):
        self.bot: BotInterface = bot_interface

        self.db = Database(self.bot.guild_id, self.bot.user_id)  # load database

        # Update the stats (this are standard for all type of messages)
        # The others stats must be set in the specific handler
        self.stats_updater = StatsUpdater(self.db)
        self.stats_updater.timestamp(self.bot.timestamp)
        self.stats_updater.username(self.bot.username)
        self.stats_updater.guild_name(self.bot.guild_name)

    @property
    def is_bot_disabled(self):
        """
        if the guild is disabled,
        this means that the bot will ignore all the messages from that guild
        :return: the bot disable status
        """
        return self.db.guild.bot_disabled

    def check_spam(self):
        if self.stats_updater.is_spam:
            self.bot.reply_text(spam_detected_reply(self.db.guild.languages[0]))

    def check_level_up(self):
        """
        Check if the user has reached the xp to level up.
        If yes, send the level up chard in the same chat.
        The level up card has text, so in must be formatted based on the guild language
        """
        # check if the bot is paused in that guild
        if self.db.guild.bot_paused:
            return

        if self.stats_updater.is_level_up:
            data = {
                'level': self.db.user.guild.xp.level,
                'bold_text': user_field(self.db.guild.languages[0], self.bot.username),
                'text': text_description(self.db.guild.languages[0], self.db.user.guild.xp.level),
            }
            d = DrawLevelUpCard(data)
            d.draw_level_up()

            # Have level_up notifications enabled? then send in te chat
            if self.db.guild.level_up_notification:
                self.bot.reply_photo(d.get_image())
            # else send it directly to the user
            else:
                self.bot.send_photo(self.bot.user_id, d.get_image())

    def check_commands(self, text):
        """
        Check if the text is a command.
        If yes execute it
        Here will be check:
        - the prefix handling (based on guild)
        - user permission to run the command
        - error handling, message error sending to log chat

        :param text: the text where search the command
        """

        if text.startswith('.'):
            log_message = 'User {} in {} sent "{}"'.format(
                    self.bot.username,
                    self.bot.guild_name,
                    text,
            )
            logger.info(log_message)

            try:
                # text without the command invocation word, and the language of the command
                command = com.find_command(text[1:], self.db.guild.languages[0], self.db.user.guild.permissions)

                user_command_use = self.db.user.guild.commands.command(command)
                if user_command_use:
                    com.hourly_executions = user_command_use.log_by_hour[0][1]
                    com.daily_executions = user_command_use.log_by_day[0][1]

                # Here you have to pass the bot object and the update obj that will be used
                out = com.run_command(self.bot)
                self.db.user.guild.commands.increment_command_interactions(command, self.bot.timestamp)

                if out:
                    # must be sent with parse mode enabled
                    self.bot.reply_text(out, parse_mode=True)

            except CommandExecutionFail as exc:
                # error of the command, send to a log chat
                self.bot.reply_text(str(exc))  # the exception to send in chat
                logger.error(exc.error_report)
                self.bot.send_text(LOGGING_CHAT, '{}\n\n{}'.format(log_message, exc.error_report))

            except CommandNotFound as exc:
                self.bot.reply_text(str(exc))
                self.bot.send_text(LOGGING_CHAT, '{}\nCommand Not Found'.format(log_message))

            except ArgVoidNotAllowed as exc:
                self.bot.reply_text(str(exc))
                self.bot.send_text(LOGGING_CHAT, '{}\nArg Void Not Allowed'.format(log_message))

            except ParamNotFound as exc:
                self.bot.reply_text(str(exc))
                self.bot.send_text(LOGGING_CHAT, '{}\nParam Not Found'.format(log_message))

            except MaxHourlyUses as exc:
                self.bot.reply_text(str(exc))
                self.bot.send_text(LOGGING_CHAT, '{}\nUser has exceed MaxHourlyUses'.format(log_message))

            except MaxDailyUses as exc:
                self.bot.reply_text(str(exc))
                self.bot.send_text(LOGGING_CHAT, '{}\nUser has exceed MaxDailyUses'.format(log_message))

            self.bot.send_text(LOGGING_CHAT, '{}\nCommand Executed'.format(log_message))

            return True
        return False



from time import time


class ExecutionTime(object):
    def __init__(self):
        self.times = []

    def start_time_calculation(self, scope):
        t1 = time()
        self.times.append((scope, t1))

    def stop_time_calculation(self, scope, message=''):
        t2 = time()

        t1 = None
        for s, t in self.times:
            if s == scope:
                t1 = t
        if t1:
            t = (t2 - t1) * 1000
            print('{} executed in {} ms'.format(message, int(t)))
        else:
            print('Time with {} scope not found'.format(scope))



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

packages = find_packages(exclude=['tests*'])

setup(
    name='pvlv',
    version='0.0.1',
    license='LGPLv3',

    author='AbbestiaDC',
    author_email='not.simone.scuola@outlook.com',
    description='Bot Framework for Pavlov',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/The-Pavlov-Project/pvlv',

    packages=packages,
    include_package_data=True,

    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],

    python_requires='>=3.6',
    install_requires=[
        'pvlv-database',
        'pvlv-commando',
        'pvlv-img_builder',
        'SpeechRecognition==3.8.1',
        'pydub==0.23.1',
    ],
)



from pvlv.handlers.base_handler import BaseHandler


class ImgHandler(BaseHandler):
    """
    It will handle the operations on images.
    """
    def __init__(self, bot_interface):
        super().__init__(bot_interface)

    def handle(self, img=None, text=''):

        self.stats_updater.update_image()  # update stats

        if text:
            self.stats_updater.update_text(text)
            self.check_commands(text)
        self.check_level_up()



from threading import Thread
from pvlv.handlers.base_handler import BaseHandler
from pvlv.handlers.speech_to_text.speech_to_text import speech_to_text


class VocalHandler(BaseHandler):
    """
    It will handle the operations on the text.
    """
    def __init__(self, bot_interface):
        super().__init__(bot_interface)

    def handle(self, file, duration):

        # do fast stuff
        self.stats_updater.update_vocal(duration)  # update stats
        self.check_level_up()

        # if the bot is paused don't do stt conversions.
        if not self.db.guild.bot_paused:
            # run the speech to text on another thread, cause it's time consuming operation
            t = Thread(target=speech_to_text, args=(self.bot, file, self.bot.guild_id, self.db.guild.languages[0]))
            t.start()
